@@ -57,7 +57,7 @@ struct _mbfl_convert_filter {
 	const mbfl_encoding *from;
 	const mbfl_encoding *to;
 	int illegal_mode;
-	int illegal_substchar;
+	uint32_t illegal_substchar;
 	size_t num_illegalchar;
 	void *opaque;
 };
