@@ -1,13 +1,11 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -20,10 +18,7 @@
 #define PHP_SESSION_H
 
 #include "ext/standard/php_var.h"
-
-#if defined(HAVE_HASH_EXT) && !defined(COMPILE_DL_HASH)
-# include "ext/hash/php_hash.h"
-#endif
+#include "ext/hash/php_hash.h"
 
 #define PHP_SESSION_API 20161017
 
@@ -31,7 +26,6 @@
 #define PHP_SESSION_VERSION PHP_VERSION
 
 /* save handler macros */
-#define PS_NUM_APIS      9
 #define PS_OPEN_ARGS     void **mod_data, const char *save_path, const char *session_name
 #define PS_CLOSE_ARGS    void **mod_data
 #define PS_READ_ARGS     void **mod_data, zend_string *key, zend_string **val, zend_long maxlifetime
@@ -44,29 +38,29 @@
 
 typedef struct ps_module_struct {
 	const char *s_name;
-	int (*s_open)(PS_OPEN_ARGS);
-	int (*s_close)(PS_CLOSE_ARGS);
-	int (*s_read)(PS_READ_ARGS);
-	int (*s_write)(PS_WRITE_ARGS);
-	int (*s_destroy)(PS_DESTROY_ARGS);
+	zend_result (*s_open)(PS_OPEN_ARGS);
+	zend_result (*s_close)(PS_CLOSE_ARGS);
+	zend_result (*s_read)(PS_READ_ARGS);
+	zend_result (*s_write)(PS_WRITE_ARGS);
+	zend_result (*s_destroy)(PS_DESTROY_ARGS);
 	zend_long (*s_gc)(PS_GC_ARGS);
 	zend_string *(*s_create_sid)(PS_CREATE_SID_ARGS);
-	int (*s_validate_sid)(PS_VALIDATE_SID_ARGS);
-	int (*s_update_timestamp)(PS_UPDATE_TIMESTAMP_ARGS);
+	zend_result (*s_validate_sid)(PS_VALIDATE_SID_ARGS);
+	zend_result (*s_update_timestamp)(PS_UPDATE_TIMESTAMP_ARGS);
 } ps_module;
 
 #define PS_GET_MOD_DATA() *mod_data
 #define PS_SET_MOD_DATA(a) *mod_data = (a)
 
-#define PS_OPEN_FUNC(x) 	int ps_open_##x(PS_OPEN_ARGS)
-#define PS_CLOSE_FUNC(x) 	int ps_close_##x(PS_CLOSE_ARGS)
-#define PS_READ_FUNC(x) 	int ps_read_##x(PS_READ_ARGS)
-#define PS_WRITE_FUNC(x) 	int ps_write_##x(PS_WRITE_ARGS)
-#define PS_DESTROY_FUNC(x) 	int ps_delete_##x(PS_DESTROY_ARGS)
+#define PS_OPEN_FUNC(x) 	zend_result ps_open_##x(PS_OPEN_ARGS)
+#define PS_CLOSE_FUNC(x) 	zend_result ps_close_##x(PS_CLOSE_ARGS)
+#define PS_READ_FUNC(x) 	zend_result ps_read_##x(PS_READ_ARGS)
+#define PS_WRITE_FUNC(x) 	zend_result ps_write_##x(PS_WRITE_ARGS)
+#define PS_DESTROY_FUNC(x) 	zend_result ps_delete_##x(PS_DESTROY_ARGS)
 #define PS_GC_FUNC(x) 		zend_long ps_gc_##x(PS_GC_ARGS)
 #define PS_CREATE_SID_FUNC(x)	zend_string *ps_create_sid_##x(PS_CREATE_SID_ARGS)
-#define PS_VALIDATE_SID_FUNC(x)	int ps_validate_sid_##x(PS_VALIDATE_SID_ARGS)
-#define PS_UPDATE_TIMESTAMP_FUNC(x) 	int ps_update_timestamp_##x(PS_UPDATE_TIMESTAMP_ARGS)
+#define PS_VALIDATE_SID_FUNC(x)	zend_result ps_validate_sid_##x(PS_VALIDATE_SID_ARGS)
+#define PS_UPDATE_TIMESTAMP_FUNC(x) 	zend_result ps_update_timestamp_##x(PS_UPDATE_TIMESTAMP_ARGS)
 
 /* Legacy save handler module definitions */
 #define PS_FUNCS(x) \
@@ -133,8 +127,8 @@ typedef struct _php_session_rfc1867_progress {
 	zend_long      update_step;
 	zend_long      next_update;
 	double    next_update_time;
-	zend_bool cancel_upload;
-	zend_bool apply_trans_sid;
+	bool cancel_upload;
+	bool apply_trans_sid;
 	size_t    content_length;
 
 	zval      data;                 /* the array exported to session data */
@@ -153,58 +147,58 @@ typedef struct _php_ps_globals {
 	zend_long cookie_lifetime;
 	char *cookie_path;
 	char *cookie_domain;
-	zend_bool  cookie_secure;
-	zend_bool  cookie_httponly;
+	bool  cookie_secure;
+	bool  cookie_httponly;
 	char *cookie_samesite;
 	const ps_module *mod;
 	const ps_module *default_mod;
 	void *mod_data;
 	php_session_status session_status;
+	zend_string *session_started_filename;
+	uint32_t session_started_lineno;
 	zend_long gc_probability;
 	zend_long gc_divisor;
 	zend_long gc_maxlifetime;
 	int module_number;
 	zend_long cache_expire;
-	union {
-		zval names[PS_NUM_APIS];
-		struct {
-			zval ps_open;
-			zval ps_close;
-			zval ps_read;
-			zval ps_write;
-			zval ps_destroy;
-			zval ps_gc;
-			zval ps_create_sid;
-			zval ps_validate_sid;
-			zval ps_update_timestamp;
-		} name;
+	struct {
+		zval ps_open;
+		zval ps_close;
+		zval ps_read;
+		zval ps_write;
+		zval ps_destroy;
+		zval ps_gc;
+		zval ps_create_sid;
+		zval ps_validate_sid;
+		zval ps_update_timestamp;
 	} mod_user_names;
-	int mod_user_implemented;
-	int mod_user_is_open;
+	bool mod_user_implemented;
+	bool mod_user_is_open;
+	zend_string *mod_user_class_name;
 	const struct ps_serializer_struct *serializer;
 	zval http_session_vars;
-	zend_bool auto_start;
-	zend_bool use_cookies;
-	zend_bool use_only_cookies;
-	zend_bool use_trans_sid; /* contains the INI value of whether to use trans-sid */
+	bool auto_start;
+	bool use_cookies;
+	bool use_only_cookies;
+	bool use_trans_sid; /* contains the INI value of whether to use trans-sid */
 
 	zend_long sid_length;
 	zend_long sid_bits_per_character;
-	int send_cookie;
-	int define_sid;
+	bool send_cookie;
+	bool define_sid;
 
 	php_session_rfc1867_progress *rfc1867_progress;
-	zend_bool rfc1867_enabled; /* session.upload_progress.enabled */
-	zend_bool rfc1867_cleanup; /* session.upload_progress.cleanup */
+	bool rfc1867_enabled; /* session.upload_progress.enabled */
+	bool rfc1867_cleanup; /* session.upload_progress.cleanup */
 	char *rfc1867_prefix;  /* session.upload_progress.prefix */
 	char *rfc1867_name;    /* session.upload_progress.name */
 	zend_long rfc1867_freq;         /* session.upload_progress.freq */
 	double rfc1867_min_freq;   /* session.upload_progress.min_freq */
 
-	zend_bool use_strict_mode; /* whether or not PHP accepts unknown session ids */
-	zend_bool lazy_write; /* omit session write when it is possible */
-	zend_bool in_save_handler; /* state if session is in save handler or not */
-	zend_bool set_handler;     /* state if session module i setting handler or not */
+	bool use_strict_mode; /* whether or not PHP accepts unknown session ids */
+	bool lazy_write; /* omit session write when it is possible */
+	bool in_save_handler; /* state if session is in save handler or not */
+	bool set_handler;     /* state if session module i setting handler or not */
 	zend_string *session_vars; /* serialized original session data */
 } php_ps_globals;
 
@@ -228,7 +222,7 @@ ZEND_TSRMLS_CACHE_EXTERN()
 typedef struct ps_serializer_struct {
 	const char *name;
 	zend_string *(*encode)(PS_SERIALIZER_ENCODE_ARGS);
-	int (*decode)(PS_SERIALIZER_DECODE_ARGS);
+	zend_result (*decode)(PS_SERIALIZER_DECODE_ARGS);
 } ps_serializer;
 
 #define PS_SERIALIZER_ENCODE_NAME(x) ps_srlzr_encode_##x
@@ -237,7 +231,7 @@ typedef struct ps_serializer_struct {
 #define PS_SERIALIZER_ENCODE_FUNC(x) \
 	zend_string *PS_SERIALIZER_ENCODE_NAME(x)(PS_SERIALIZER_ENCODE_ARGS)
 #define PS_SERIALIZER_DECODE_FUNC(x) \
-	int PS_SERIALIZER_DECODE_NAME(x)(PS_SERIALIZER_DECODE_ARGS)
+	zend_result PS_SERIALIZER_DECODE_NAME(x)(PS_SERIALIZER_DECODE_ARGS)
 
 #define PS_SERIALIZER_FUNCS(x) \
 	PS_SERIALIZER_ENCODE_FUNC(x); \
@@ -249,31 +243,30 @@ typedef struct ps_serializer_struct {
 /* default create id function */
 PHPAPI zend_string *php_session_create_id(PS_CREATE_SID_ARGS);
 /* Dummy PS module functions */
-PHPAPI int php_session_validate_sid(PS_VALIDATE_SID_ARGS);
-PHPAPI int php_session_update_timestamp(PS_UPDATE_TIMESTAMP_ARGS);
+PHPAPI zend_result php_session_validate_sid(PS_VALIDATE_SID_ARGS);
+PHPAPI zend_result php_session_update_timestamp(PS_UPDATE_TIMESTAMP_ARGS);
 
-PHPAPI void session_adapt_url(const char *, size_t, char **, size_t *);
+PHPAPI void session_adapt_url(const char *url, size_t url_len, char **new_url, size_t *new_len);
 
-PHPAPI int php_session_destroy(void);
+PHPAPI zend_result php_session_destroy(void);
 PHPAPI void php_add_session_var(zend_string *name);
 PHPAPI zval *php_set_session_var(zend_string *name, zval *state_val, php_unserialize_data_t *var_hash);
 PHPAPI zval *php_get_session_var(zend_string *name);
 
-PHPAPI int php_session_register_module(const ps_module *);
+PHPAPI zend_result php_session_register_module(const ps_module *);
 
-PHPAPI int php_session_register_serializer(const char *name,
+PHPAPI zend_result php_session_register_serializer(const char *name,
 	        zend_string *(*encode)(PS_SERIALIZER_ENCODE_ARGS),
-	        int (*decode)(PS_SERIALIZER_DECODE_ARGS));
+	        zend_result (*decode)(PS_SERIALIZER_DECODE_ARGS));
 
-PHPAPI void php_session_set_id(char *id);
-PHPAPI int php_session_start(void);
-PHPAPI int php_session_flush(int write);
+PHPAPI zend_result php_session_start(void);
+PHPAPI zend_result php_session_flush(int write);
 
-PHPAPI const ps_module *_php_find_ps_module(char *name);
-PHPAPI const ps_serializer *_php_find_ps_serializer(char *name);
+PHPAPI const ps_module *_php_find_ps_module(const char *name);
+PHPAPI const ps_serializer *_php_find_ps_serializer(const char *name);
 
-PHPAPI int php_session_valid_key(const char *key);
-PHPAPI int php_session_reset_id(void);
+PHPAPI zend_result php_session_valid_key(const char *key);
+PHPAPI zend_result php_session_reset_id(void);
 
 #define PS_ADD_VARL(name) do {										\
 	php_add_session_var(name);							\
@@ -297,7 +290,7 @@ PHPAPI int php_session_reset_id(void);
 	HashTable *_ht = Z_ARRVAL_P(Z_REFVAL(PS(http_session_vars)));	\
 	ZEND_HASH_FOREACH_KEY(_ht, num_key, key) {						\
 		if (key == NULL) {											\
-			php_error_docref(NULL, E_NOTICE,						\
+			php_error_docref(NULL, E_WARNING,						\
 					"Skipping numeric key " ZEND_LONG_FMT, num_key);\
 			continue;												\
 		}															\
@@ -311,16 +304,12 @@ PHPAPI ZEND_EXTERN_MODULE_GLOBALS(ps)
 
 void php_session_auto_start(void *data);
 
-#define PS_CLASS_NAME "SessionHandler"
 extern PHPAPI zend_class_entry *php_session_class_entry;
 
-#define PS_IFACE_NAME "SessionHandlerInterface"
 extern PHPAPI zend_class_entry *php_session_iface_entry;
 
-#define PS_SID_IFACE_NAME "SessionIdInterface"
 extern PHPAPI zend_class_entry *php_session_id_iface_entry;
 
-#define PS_UPDATE_TIMESTAMP_IFACE_NAME "SessionUpdateTimestampHandlerInterface"
 extern PHPAPI zend_class_entry *php_session_update_timestamp_iface_entry;
 
 extern PHP_METHOD(SessionHandler, open);
@@ -330,7 +319,5 @@ extern PHP_METHOD(SessionHandler, write);
 extern PHP_METHOD(SessionHandler, destroy);
 extern PHP_METHOD(SessionHandler, gc);
 extern PHP_METHOD(SessionHandler, create_sid);
-extern PHP_METHOD(SessionHandler, validateId);
-extern PHP_METHOD(SessionHandler, updateTimestamp);
 
 #endif
